@@ -73,7 +73,6 @@ def find_possible_first_note(music):
     to parse the music.
     """
     i = 0
-    v = music.split()
     # FIXME regexes are modified copies from the mpd Lexer. Try to reuse
     # code in the future.
     re_white = re.compile(r"\s+")
@@ -109,7 +108,7 @@ def validate_only_notenames(s):
     while 1:
         try:
             toc, toc_data = lex.get()
-        except InvalidNotenameException, e:
+        except InvalidNotenameException:
             return lex.get_error_location()
         if not toc:
             break
