@@ -18,6 +18,9 @@
 
 # This script is used to launch Solfege when running
 # it from the source dir without installing.
+import __builtin__
+import time
+__builtin__.start_time = time.time()
 
 import sys
 import os
@@ -45,9 +48,9 @@ if sys.platform == 'win32':
         print "Migration failed:", e
 
 if not os.path.exists(filesystem.app_data()):
-    os.mkdir(filesystem.app_data())
+    os.makedirs(filesystem.app_data())
 if not os.path.exists(filesystem.user_data()):
-    os.mkdir(filesystem.user_data())
+    os.makedirs(filesystem.user_data())
 
 try:
     src.cfg.initialise("default.config", None, filesystem.rcfile())
