@@ -23,12 +23,13 @@ import soundcard, mpd
 
 import utils
 import random
-import locale
 import sys
 import traceback
 import cfg
 import lessonfile
 import osutils
+import const
+import statistics
 
 class QstatusDefs:
     QSTATUS_NO = 0
@@ -78,7 +79,8 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         self.m_lessonfile = lessonfile
         self.parse_lessonfile()
         if self.m_P and self.m_statistics:
-            self.m_statistics.lessonfile_changed()
+            statistics.db.validate_stored_statistics(
+                self.m_P.header.lesson_id, self.m_P.m_filename)
     def parse_lessonfile(self):
         self.m_question = None
         self.q_status = self.QSTATUS_NO
@@ -152,7 +154,6 @@ class MelodicIntervalTeacher(Teacher):
                     break
                 if old_tonika != self.m_tonika and self.m_tonika + self.m_question[0] != old_toptone:
                     break
-                print "try again."
             self.q_status = self.QSTATUS_NEW
             return self.OK
 
@@ -219,24 +220,6 @@ class MelodicIntervalTeacher(Teacher):
 
 
 class RhythmAddOnClass:
-    RHYTHMS = ("c4", "c8 c8", "c16 c16 c16 c16", "c8 c16 c16",
-               "c16 c16 c8", "c16 c8 c16", "c8. c16", "c16 c8.",
-               "r4", "r8c8", "r8 c16 c16", "r16 c16 c8", "r16c8c16",
-               "r16 c16 c16 c16", "r8 r16 c16", "r16 c8.",
-               "c12 c12 c12", "r12 c12 c12",
-               "c12 r12 c12", "c12 c12 r12", "r12 r12 c12", "r12 c12 r12",
-               "c4.", "c4 c8", # 22, 23
-               "c8 c4", "c8 c8 c8", # 24, 25
-               "c4 c16 c16", # 26
-               "c16 c16 c4", # 27
-               "c8 c8 c16 c16", #28
-               "c8 c16 c16 c8", #29
-               "c16 c16 c8 c8", #30
-               "c8 c16 c16 c16 c16", #31
-               "c16 c16 c8 c16 c16", #32
-               "c16 c16 c16 c16 c8", #33
-               "c16 c16 c16 c16 c16 c16", #34
-               )
     def new_question(self):
         """returns:
                self.ERR_PICKY : if the question is not yet solved and the
@@ -258,7 +241,7 @@ class RhythmAddOnClass:
         norest_v = []
         v = []
         for x in self.m_P.header.rhythm_elements:
-            if not (self.RHYTHMS[x][0] == "r"
+            if not (const.RHYTHMS[x][0] == "r"
                     and self.get_bool("not_start_with_rest")):
                 norest_v.append(x)
             v.append(x)
@@ -283,7 +266,7 @@ class RhythmAddOnClass:
             else:
                 count_in_notelen = "4"
             s = "d%s " % count_in_notelen * self.get_int("count_in")
-        s += " ".join([self.RHYTHMS[k] for k in self.m_question])
+        s += " ".join([const.RHYTHMS[k] for k in self.m_question])
         return s
     def get_music_string(self):
         """
@@ -401,9 +384,9 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
     def on_start_practise(self):
         """Dummy function that child classes can override.
         """
-        print "dummy abstract.Gui.on_start_practise", self.m_exname
+        pass
     def on_end_practise(self):
-        print "dummy abstract.Gui.on_end_practise", self.m_exname
+        pass
     def handle_config_box_visibility(self):
         """
         Show self.config_box if it has any visible children, otherwise
@@ -572,7 +555,7 @@ class RhythmAddOnGuiClass(object):
         table.show_all()
     def pngcheckbutton(self, i):
         btn = gtk.CheckButton()
-        btn.add(gu.create_rhythm_image(RhythmAddOnClass.RHYTHMS[i]))
+        btn.add(gu.create_rhythm_image(const.RHYTHMS[i]))
         btn.show()
         btn.connect('clicked', self.select_element_cb, i)
         return btn
@@ -727,9 +710,9 @@ class LessonbasedGui(Gui):
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             self.m_t.m_P.header.random_transpose = dlg.get_value()
-            if self.m_t.m_P.header.random_transpose == True:
+            if self.m_t.m_P.header.random_transpose:
                 self.g_random_transpose.set_text(_("Yes"))
-            elif self.m_t.m_P.header.random_transpose == False:
+            elif not self.m_t.m_P.header.random_transpose:
                 self.g_random_transpose.set_text(_("No"))
             else:
                 self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
