@@ -14,25 +14,23 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import buildinfo
+import logging
 import errno, sys
 import subprocess
 import locale
 import tempfile
 import shutil
-import optparse
+import gobject
 import soundcard, mpd
 import lessonfile
 import dataparser
 import os
-import re
 import cfg, utils
 import webbrowser
 import gu
-from osutils import *
 import reportlib
-import filesystem
 import osutils
+import statistics
 
 def check_rcfile():
     """See default.config for rcfileversion values, meanings and
@@ -121,47 +119,6 @@ def check_rcfile():
             cfg.set_string("user/lowest_pitch", "c'")
 
 
-class SolfegeOptionParser(optparse.OptionParser):
-    def __init__(self):
-        optparse.OptionParser.__init__(self)
-        self.add_option('-v', '--version', action='store_true', dest='version')
-        self.add_option('-w', '--warranty', action='store_true', dest='warranty',
-            help=_('Show warranty and copyright.'))
-        self.add_option('--no-splash', action='store_false', dest='no_splash',
-            help=_('Do not show the startup window.'),
-            default=False)
-        self.add_option('--verbose-sound-init', action='store_true',
-            default=False,
-            dest='verbose_sound_init',
-            help=_('Display more info about the sound setup.'))
-        self.add_option('--no-sound', action='store_true', dest='no_sound',
-            default=False,
-            help=_('Do not play any sounds. Instead some data is printed to standard output. Use this for debugging and porting.'))
-        self.add_option('--debug', action='store_true', dest='debug',
-            help=_('Include features used by the Solfege developers to debug the program.'))
-        self.add_option('--disable-exception-handler', action='store_true',
-            dest='disable_exception_handler',
-            help=_("Disable the exception handling in Gui.standard_exception_handler."))
-        self.add_option('--no-random', action='store_true', dest='no_random',
-            help=_('For debugging only: Select questions from lesson files in sequential order.'))
-        self.add_option('--enable-gtkhtml', action='store_true', 
-            dest='enable_gtkhtml',
-            help=_('Run using gtkhtml2 instead of the built in HTML viewer.'))
-        self.add_option('--no-cairo-widgets', action='store_true',
-            dest='no_cairo_widgets',
-            help=_("Do not use the cairo version of input widgets, even if we run gtk+ 2.8.0 or newer."))
-        self.add_option('--show-gtk-warnings', action='store_true',
-            dest='show_gtk_warnings',
-            help=_('Show GtkWarnings and PangoWarnings in the traceback window.'))
-    def print_help(self, outfile=None):
-        if outfile is None:
-            outfile = sys.stdout
-        encoding = outfile.encoding
-        if not encoding:
-            encoding = "iso-8859-1"
-        outfile.write(self.format_help().encode(encoding, 'replace'))
-
-
 class SolfegeApp(cfg.ConfigUtils):
     def __init__(self, options, ui, lessonfile_manager):
         """
@@ -177,8 +134,39 @@ class SolfegeApp(cfg.ConfigUtils):
         self.m_teachers = {}
         self.m_running_exercise = None
         self.lessonfile_manager = lessonfile_manager
+        self.m_frontpage_data = eval(open("startpage.py-data").read(),
+                {'_': lambda s: s}, {'_': lambda s: s})
         self.m_sound_init_exception = None
         self.setup_sound()
+        # We have to check if lesson files have changed since last time we practised.
+        self._pp = lessonfile_manager.parse(options.debug)
+        gobject.idle_add(self.reparse_lessonfiles)
+    def reparse_lessonfiles(self):
+        """
+        Parse all lesson files when we are idle.
+        """
+        try:
+            uuid, data = self._pp.next()
+            return True
+        except StopIteration:
+            self.m_ui.handle_lesson_id_crash()
+            self._pp = lessonfile.manager.iterate_lesson_ids()
+            gobject.idle_add(self.check_for_changed_lessonfiles)
+            return False
+    def check_for_changed_lessonfiles(self):
+        """
+        Feed the parsed lesson file data to the database. We do
+        it when we are idle.
+        """
+        try:
+            uuid = self._pp.next()
+            data = lessonfile.manager.m_uiddb[uuid]
+            statistics.db.set_lessonfile_data(uuid, data['header']['title'],
+                    data['header']['module'], data['filename'])
+            return True
+        except StopIteration:
+            del self._pp
+        return False
     def setup_sound(self):
         if sys.platform == 'win32' and \
                     cfg.get_string("sound/type") == "sequencer-device":
@@ -214,7 +202,8 @@ class SolfegeApp(cfg.ConfigUtils):
             soundcard.synth.error_report_cb = self.m_ui.display_error_message
         elif cfg.get_string("sound/type") == '':
             self.m_ui.display_error_message(
-_("You should configure sound from the 'Sound' page of the preferences window."))
+                _("You should configure sound from the 'Sound' page "
+                  "of the preferences window."))
         elif cfg.get_string("sound/type") == "sequencer-device":
             try:
                 soundcard.initialise_devicefile(
@@ -245,28 +234,26 @@ _("You should configure sound from the 'Sound' page of the preferences window.")
             self.m_ui.display_error_message(
             """%s""" % str(e).decode(locale.getpreferredencoding(), 'replace'))
         elif isinstance(e, ImportError):
-            estr = str(e).decode(locale.getpreferredencoding(), 'replace')
             self.m_ui.display_error_message2(str(e), _("You should configure sound from the preferences window, and try to use an external midi player. Or try to recompile the program and check for error messages to see why the module is not built."))
         elif e.errno == errno.EACCES:
             self.m_ui.display_error_message(
-"""The sound init failed: %s
-The errno EACCES indicates that you don't have write
-permission to the device.""" % str(e).decode(locale.getpreferredencoding(), 'replace'))
+                "The sound init failed: %s\n"
+                "The errno EACCES indicates that you don't have write "
+                "permission to the device."
+                % str(e).decode(locale.getpreferredencoding(), 'replace'))
         elif e.errno == errno.EBUSY:
             self.m_ui.display_error_message(
-"""The sound init failed: %s
-It seems like some other program is using the device. You
-should try to quit that other program and restart Solfege."""
- % str(e).decode(locale.getpreferredencoding(), 'replace'))
+                "The sound init failed: %s\n"
+                "It seems like some other program is using the device. You "
+                "should try to quit that other program and restart Solfege."
+                % str(e).decode(locale.getpreferredencoding(), 'replace'))
         else:
             self.m_ui.display_error_message(
-"""The sound init failed: 
-    %s
-You should configure sound from the 'Sound' page of the preferences window.
-
-It is also possible that the OS sound setup is incorrect.
-"""
- % str(e).decode(locale.getpreferredencoding(), 'replace'))
+                "The sound init failed: %s\n"
+                "You should configure sound from the 'Sound' page of "
+                "the preferences window.\n\n"
+                "It is also possible that the OS sound setup is incorrect."
+                % str(e).decode(locale.getpreferredencoding(), 'replace'))
     def play_happy_sound(self):
         mpd.play_music(r"\staff\relative c'{c16 e g a}", 180, 8,
                cfg.get_int('config/preferred_instrument_volume'))
@@ -288,7 +275,7 @@ It is also possible that the OS sound setup is incorrect.
         """
         return the module name.
         """
-        module = self.lessonfile_manager.get(lesson_id, 'module')
+        module = statistics.db.get_field(lesson_id, 'module')
         if self.m_running_exercise:
             self.m_ui.box_dict[self.m_running_exercise].on_end_practise()
         if module not in self.m_teachers:
@@ -296,7 +283,7 @@ It is also possible that the OS sound setup is incorrect.
         if module not in self.m_ui.box_dict:
             self.m_ui.initialise_exercise(self.m_teachers[module])
         self.m_teachers[module].set_lessonfile(
-            self.lessonfile_manager.get(lesson_id, 'filename'))
+            statistics.db.get_field(lesson_id, 'filename'))
         self.m_ui.activate_exercise(module, urlobj)
         self.m_running_exercise = module
         self.m_teachers[module].g_view = self.m_ui.box_dict[module]
@@ -307,8 +294,9 @@ It is also possible that the OS sound setup is incorrect.
             module = self._practise_lesson_id(lesson_id)
         except (lessonfile.LessonfileParseException,
                 dataparser.DataparserException), e:
-            self.m_ui.display_exception_message(e, lessonfile=self.lessonfile_manager.get(lesson_id, 'filename'))
-            module = self.lessonfile_manager.get(lesson_id, 'module')
+            self.m_ui.display_exception_message(
+                e, lessonfile=statistics.db.get_field(lesson_id, 'filename'))
+            module = statistics.db.get_field(lesson_id, 'module')
             self.m_teachers[module].m_P = None
         if 'm_discards' in dir(self.m_teachers[module].m_P):
             for msg in self.m_teachers[module].m_P.m_discards:
@@ -371,7 +359,7 @@ It is also possible that the OS sound setup is incorrect.
             else:
                 gu.dialog_ok(_("You have not selected email program. You can do so in the preferences window."))
         else:
-            print "unknown link type", urlobj.protocol
+            gu.dialog_ok("unknown link type: %s" % urlobj.protocol)
     def handle_solfege_href(self, urlobj):
         """
         This method handles three kins of urls:
@@ -428,7 +416,7 @@ It is also possible that the OS sound setup is incorrect.
             cfg.sync()
         except IOError, e:
             self.m_ui.display_exception_message(e)
-        if self.m_ui.m_viewer != 'docviewer':
+        if self.m_ui.m_viewer not in ('docviewer', 'exerciseviewer', 'frontpage'):
             self.m_ui.box_dict[self.m_ui.m_viewer].on_end_practise()
         if soundcard.synth:
             soundcard.synth.close()
@@ -457,11 +445,11 @@ It is also possible that the OS sound setup is incorrect.
         report.append(table)
         for lesson_info in export_data:
             lesson_id = lesson_info['lesson_id']
-            module = self.lessonfile_manager.get(lesson_id, 'module')
+            module = statistics.db.get_field(lesson_id, 'module')
             if module not in self.m_teachers:
                 self.create_teacher(module)
             p = self.m_teachers[module].lessonfileclass()
-            p.parse_file(self.lessonfile_manager.get(lesson_id, 'filename'))
+            p.parse_file(statistics.db.get_field(lesson_id, 'filename'))
             for c in range(lesson_info['count']):
                 if module == 'idbyname':
                     p.select_random_question()
@@ -482,7 +470,7 @@ It is also possible that the OS sound setup is incorrect.
                     soundcard.end_export()
                 elif module in ('melodicinterval', 'harmonicinterval'):
                     t = self.m_teachers[module]
-                    t.set_lessonfile(self.lessonfile_manager.get(lesson_id, 'filename'))
+                    t.set_lessonfile(statistics.db.get_field(lesson_id, 'filename'))
                     t.start_practise()
                     t.new_question("c", "c''")
                     t.q_status = t.QSTATUS_SOLVED
@@ -500,7 +488,7 @@ It is also possible that the OS sound setup is incorrect.
                                     (self.get_int('config/default_bpm'), 4))
                     soundcard.end_export()
                 else:
-                    print "ignoring exercise with module='%s'" % module
+                    logging.warning("export_training_set:ignoring exercise with module='%s'" % module)
 #####
                 def do_convert(from_format, to_format):
                     """
@@ -563,11 +551,11 @@ It is also possible that the OS sound setup is incorrect.
                  and not sdict['count'] because sometimes the app has some
                  questions, and just need a few more.
         """
-        module = self.lessonfile_manager.get(sdict['lesson_id'], 'module')
+        module = statistics.db.get_field(sdict['lesson_id'], 'module')
         if module not in self.m_teachers:
             self.create_teacher(module)
         p = self.m_teachers[module].lessonfileclass()
-        p.parse_file(self.lessonfile_manager.get(sdict['lesson_id'], 'filename'))
+        p.parse_file(statistics.db.get_field(sdict['lesson_id'], 'filename'))
         if module == 'idbyname':
             for x in self._sheet_gen_question_idbyname(p, count, sdict):
                 yield x
@@ -607,7 +595,7 @@ It is also possible that the OS sound setup is incorrect.
         # has been selected, so get an even selection. We don't do it
         # here at the moment, because we need to descide what we really want.
         teacher = self.m_teachers[module]
-        teacher.set_lessonfile(self.lessonfile_manager.get(sdict['lesson_id'],
+        teacher.set_lessonfile(statistics.db.get_field(sdict['lesson_id'],
                                                           'filename'))
         teacher.start_practise()
         for x in range(count):
