@@ -18,7 +18,7 @@ import gtk
 import gobject
 import gu
 from specialwidgets import QuestionNameCheckButtonTable
-import soundcard, mpd, mpd.musicdisplayer
+import mpd, mpd.musicdisplayer
 import abstract, lessonfile
 import soundcard
 
