@@ -18,24 +18,23 @@ import sys
 import os
 import gobject, gtk
 import gu, mpd
-import buildinfo
 import notenamespinbutton
 from instrumentselector import nInstrumentSelector, InstrumentConfigurator
 import cfg
 import osutils
 import soundcard
 import languages
-import filesystem
 import winlang
 
 if sys.platform == 'win32':
     try:
         from soundcard import winmidi
     except ImportError, e:
-        print >> sys.stderr, "Loading winmidi.pyd failed:"
-        print >> sys.stderr, e
+        print "Loading winmidi.pyd failed:"
+        print e
         winmidi = None
 
+
 class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
     def on_destroy(self, widget, e):
         self.m_app.m_ui.g_config_window.destroy()
@@ -69,8 +68,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             gtk.Adjustment(self.get_int('config/default_bpm'), 10, 500, 1, 10))
         self.g_arpeggio_bpm = gu.nSpinButton('config', 'arpeggio_bpm',
             gtk.Adjustment(self.get_int('config/arpeggio_bpm'), 10, 500, 1, 10))
-        for text, widget in [(_("_Default:") ,self.g_default_bpm), 
-                             (_("A_rpeggio:") ,self.g_arpeggio_bpm)]:
+        for text, widget in [("_Default:",self.g_default_bpm), ("A_rpeggio:",self.g_arpeggio_bpm)]:
             label = gtk.Label(text)
             label.set_alignment(0.0, 0.5)
             label.set_use_underline(True)
