@@ -513,7 +513,10 @@ class Dataparser:
             # functioncall()
             self._lexer.scan(')')
             try:
-                return self.functions[name]()
+                if self.functions[name][0]:
+                    return self.functions[name][1](self)
+                else:
+                    return self.functions[name][1]()
             except KeyError:
                 raise NameLookupException(self, npos)
         else:
@@ -521,10 +524,13 @@ class Dataparser:
             arglist = self.faktorlist()
             self._lexer.scan(')')
             try:
-                return self.functions[name](*arglist)
+                if self.functions[name][0]:
+                    return self.functions[name][1](self, *arglist)
+                else:
+                    return self.functions[name][1](*arglist)
             except KeyError:
                 raise NameLookupException(self, npos)
-            except TypeError, e:
+            except TypeError:
                 raise WrongArgumentCount(self, npos)
     def block(self):
         """block: NAME "{" assignmentlist "}" """
