@@ -20,7 +20,7 @@ import mpd, mpd.musicdisplayer
 import abstract, lessonfile
 from mpd.rat import Rat
 import soundcard
-import utils
+import cfg
 
 class Teacher(abstract.Teacher):
     """
@@ -41,7 +41,7 @@ class Gui(abstract.LessonbasedGui):
         ################
         self.g_question_title = gu.bLabel(self.practise_box, "", False, False)
 
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.set_size_request(500, -1)
         self.g_music_displayer.show()
         self.practise_box.pack_start(self.g_music_displayer)
@@ -53,7 +53,7 @@ class Gui(abstract.LessonbasedGui):
         self.g_go_back.connect('clicked', self.select_previous)
         self.g_go_back.show()
         self.action_area.pack_start(self.g_go_back, False)
-        self.g_go_forward = b = gtk.Button(stock='gtk-go-forward')
+        self.g_go_forward = gtk.Button(stock='gtk-go-forward')
         self.g_go_forward.show()
         self.g_go_forward.connect('clicked', self.select_next)
         self.action_area.pack_start(self.g_go_forward, False)
@@ -127,9 +127,10 @@ class Gui(abstract.LessonbasedGui):
         # tmp func used as callback function
         def f(w, start, end, self=self):
             try:
-                mpd.play_music3(self.m_t.m_P.get_music(),
+                mpd.play_music(self.m_t.m_P.get_music(),
                     self.m_t.m_P.get_tempo(),
-                    self.m_t.m_P.prepare_instrument_list(self.m_t.m_P.get_question()),
+                    cfg.get_int('config/preferred_instrument'),
+                    cfg.get_int('config/preferred_instrument_volume'),
                     start, end)
             except Exception, e:
                 if not self.standard_exception_handler(e, soundcard.synth.stop):
