@@ -16,7 +16,6 @@
 
 
 import sys
-import locale
 import traceback
 import gtk
 import gu
@@ -40,17 +39,24 @@ class ExceptionDialog(gtk.Dialog):
         hbox.pack_start(vbox)
         self.msg_vbox = gtk.VBox()
         vbox.pack_start(self.msg_vbox)
-        if 'args' in dir(exception) and exception.args:
-            estr = exception.args[0]
-        else:
-            estr = exception
-        if not isinstance(estr, unicode):
-            estr = str(estr).decode(sys.getfilesystemencoding(), 'replace')
-        self.g_primary = gtk.Label(estr)
+        self.g_primary = gtk.Label()
         self.g_primary.set_alignment(0.0, 0.5)
         self.g_primary.set_line_wrap(True)
         self.m_primary_bold = False
         self.msg_vbox.pack_start(self.g_primary)
+        if isinstance(exception, StandardError):
+            self.g_primary.set_text(str(exception).decode(sys.getfilesystemencoding(), 'replace'))
+        else:
+            if 'args' in dir(exception) and exception.args:
+                estr = exception.args[0]
+            else:
+                estr = exception
+            if not isinstance(estr, unicode):
+                estr = str(estr).decode(sys.getfilesystemencoding(), 'replace')
+            self.g_primary.set_text(estr)
+            if 'args' in dir(exception):
+                for s in exception.args[1:]:
+                    self.add_text(s)
         # This label is here just for spacing
         l = gtk.Label("")
         vbox.pack_start(l)
@@ -64,9 +70,9 @@ class ExceptionDialog(gtk.Dialog):
         sc.set_size_request(-1, 100)
         expander.add(sc)
         sc.add_with_viewport(l)
-        if 'args' in dir(exception):
-            for s in exception.args[1:]:
-                self.add_text(s)
+        l = gtk.Label(_("Visit http://www.solfege.org/support if you need help."))
+        l.set_alignment(0.0, 0.5)
+        vbox.pack_start(l, False)
         w = self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
         self.set_default_response(gtk.RESPONSE_CLOSE)
         self.set_focus(w)
