@@ -74,6 +74,15 @@ def _get_home_dir():
         else: return path2
     else: return path1
 
+
+def win32_program_files_folder():
+    """
+    Return the name of the C:\Program files folder.
+    """
+    return winreg._get_reg_value(winreg._winreg.HKEY_LOCAL_MACHINE,
+        r"SOFTWARE\Microsoft\Windows\CurrentVersion", r"ProgramFilesDir")
+
+
 def _win32_unicode_decode(s):
     enc = sys.getfilesystemencoding()
     if enc is None or enc == "":
@@ -99,7 +108,7 @@ def get_home_dir():
 def expanduser(s):
     return s.replace("~", get_home_dir())
 
-def user_data(dir=None):
+def user_data():
     """
     Return the full path name of a directory where solfege  by default will suggest to
     place the directories trainingsets/ eartrainingtests/ 
