@@ -17,6 +17,7 @@
 
 import time
 import gtk
+import pango
 import gobject
 import re
 import soundcard
@@ -166,23 +167,32 @@ class nCombo(gtk.Combo, cfg.ConfigUtils):
     def entry_changed(self, entry):
         self.set_int(self.m_name, self.popdown_strings.index(entry.get_text()))
 
-class PercussionNameComboBoxEntry(gtk.ComboBoxEntry, cfg.ConfigUtils):
+class PercussionInstrumentMenu(gtk.Menu):
+    def __init__(self, callback):
+        gtk.Menu.__init__(self)
+        for idx, pname in enumerate(soundcard.percussion_names):
+            menuitem = gtk.MenuItem(pname)
+            menuitem.connect('activate', callback, idx)
+            self.append(menuitem)
+            menuitem.show()
+        self.show()
+
+
+class PercussionNameButton(gtk.Button, cfg.ConfigUtils):
     def __init__(self, exname, name, default):
-        liststore = gtk.ListStore(gobject.TYPE_STRING)
-        for pn in soundcard.percussion_names:
-            liststore.append((pn,))
-        gtk.ComboBoxEntry.__init__(self, liststore)
+        gtk.Button.__init__(self)
         cfg.ConfigUtils.__init__(self, exname)
         self.m_name = name
+        self.g_menu = PercussionInstrumentMenu(self.entry_changed)
         i = self.get_int(name)
         if not i:
             i = soundcard.percussionname_to_int(default)
             self.set_int(name, i)
-        self.child.set_text(soundcard.int_to_percussionname(i))
-        self.connect("changed", self.entry_changed)
-    def entry_changed(self, widget):
-        #FIXME value 35 should be taken from soundcard module.
-        self.set_int(self.m_name, widget.get_active()+35)
+        self.set_label(soundcard.int_to_percussionname(i))
+        self.connect('clicked', lambda w: self.g_menu.popup(None, None, None, 1, 0))
+    def entry_changed(self, widget, value):
+        self.set_int(self.m_name, value + soundcard.first_percussion_int_value)
+        self.set_label(soundcard.percussion_names[value])
 
 
 class FlashBar(gtk.Frame):
@@ -271,8 +281,11 @@ def hig_dlg_vbox():
     vbox.set_border_width(12)
     return vbox
 
-def hig_category_vbox(title):
+def hig_category_vbox(title, spacing=6):
     """
+    spacing  the space to put between children. HIG say is should be 6, but
+    while packing gtk.LinkButtons there is too much space between the links
+    when packing with 6 pixels. So I added the spacing parameter.
     Return a tuple of two boxes:
     box1 -- a box containing everything including the title. Useful
             if you have to hide a category.
@@ -290,14 +303,15 @@ def hig_category_vbox(title):
     hbox.pack_start(fill, False)
     category_content_vbox = gtk.VBox()
     hbox.pack_start(category_content_vbox, True)
-    category_content_vbox.set_spacing(6)
+    category_content_vbox.set_spacing(spacing)
     vbox.show_all()
     return vbox, category_content_vbox
 
-def hig_label_widget(txt, widget, sizegroup):
+def hig_label_widget(txt, widget, sizegroup, expand=False, fill=False):
     """
     Return a box containing a label and a widget, aligned nice
-    as the HIG say we should.
+    as the HIG say we should. widget could also be a list or tuple of
+    widgets.
     """
     hbox = gtk.HBox()
     hbox.set_spacing(6)
@@ -306,10 +320,10 @@ def hig_label_widget(txt, widget, sizegroup):
     if sizegroup:
         sizegroup.add_widget(label)
     hbox.pack_start(label, False)
-    if type(widget) != type([]):
+    if not isinstance(widget, (list, tuple)):
         widget = [widget]
     for w in widget:
-        hbox.pack_start(w,False)
+        hbox.pack_start(w, fill, expand)
     label.set_use_underline(True)
     label.set_mnemonic_widget(widget[0])
     return hbox
@@ -690,3 +704,51 @@ class EditorDialogBase(gtk.Window):
         ret = decode_filename(dialog.get_filename())
         dialog.destroy()
         return ret
+
+
+class LogWindow(gtk.Window):
+    def __init__(self, parent):
+        gtk.Window.__init__(self)
+        self.set_transient_for(parent)
+        self.set_destroy_with_parent(True)
+        self.set_modal(True)
+        vbox = gtk.VBox()
+        vbox.set_spacing(4)
+        self.add(vbox)
+        self.set_default_size(630, 400)
+        self.g_textbuffer = gtk.TextBuffer()
+        self.g_textbuffer.create_tag('h1', weight=pango.WEIGHT_BOLD,
+            size=16*pango.SCALE)
+        self.g_textview = gtk.TextView(buffer=self.g_textbuffer)
+        self.g_scrolledwindow = gtk.ScrolledWindow()
+        vbox.pack_start(self.g_scrolledwindow)
+        self.g_scrolledwindow.add(self.g_textview)
+        vbox.pack_start(gtk.HSeparator(), False, False)
+        bbox = gtk.HButtonBox()
+        bbox.set_layout(gtk.BUTTONBOX_END)
+        vbox.pack_start(bbox, False, False)
+        self.g_close_button = gtk.Button(stock="gtk-close")
+        self.g_close_button.set_sensitive(False)
+        self.g_close_button.connect('clicked', lambda w: self.destroy())
+        bbox.pack_start(self.g_close_button)
+        self.show_all()
+    def run_finished(self):
+        self.g_close_button.set_sensitive(True)
+    def write(self, s, tag=None):
+        if tag:
+            self.g_textbuffer.insert_with_tags_by_name(
+                self.g_textbuffer.get_end_iter(),
+                s,
+                tag)
+        else:
+            self.g_textbuffer.insert(self.g_textbuffer.get_end_iter(), s)
+        self.g_textview.scroll_to_iter(self.g_textbuffer.get_end_iter(), 0)
+        # This is needed to make the window update:
+        while gtk.events_pending():
+            gtk.main_iteration()
+
+
+class ClickableLabel(gtk.LinkButton):
+    def __init__(self, *w):
+        gtk.LinkButton.__init__(self, *w)
+        self.get_children()[0].set_alignment(0.0, 0.5)
