@@ -18,10 +18,8 @@ import gtk, gobject
 import gu
 from specialwidgets import QuestionNameButtonTable, QuestionNameCheckButtonTable
 import abstract, mpd, mpd.musicdisplayer
-import utils
 import statistics, statisticsviewer
 import lessonfile
-import dataparser
 import soundcard
 import osutils
 
@@ -90,7 +88,7 @@ class Gui(abstract.LessonbasedGui):
         ################
         # practise_box #
         ################
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.practise_box.pack_start(self.g_music_displayer, False)
         self.g_bb = QuestionNameButtonTable(self.m_t.m_exname)
         self.practise_box.pack_start(self.g_bb, False)
