@@ -14,8 +14,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
 import gtk, gobject
-import soundcard
 import abstract, statistics, statisticsviewer, gu, random
 import lessonfile
 import utils
@@ -94,7 +94,7 @@ class Teacher(abstract.Teacher):
         self.q_status = self.QSTATUS_NO
     def give_up(self):
         self.stop_question()
-        print "FIXME not saving statistics"
+        logging.debug("identifybpm.give_up:FIXME not saving statistics")
         self.q_status = self.QSTATUS_GIVE_UP
 
 class Gui(abstract.Gui):
@@ -138,8 +138,6 @@ class Gui(abstract.Gui):
         ########
         self.g_notebook.get_nth_page(1).hide()
         self.update_buttons()
-    def on_enable_bpm(self, _o=None):
-        print "enable", _o
     def on_event(self, button, event):
         if event.type == gtk.gdk.BUTTON_RELEASE and event.button == 3:
             self.m_t.toggle_active(button.get_data('bpm'))
