@@ -18,7 +18,7 @@ import gtk
 import gobject
 import gu
 from specialwidgets import QuestionNameCheckButtonTable
-import soundcard, mpd, mpd.musicdisplayer
+import mpd, mpd.musicdisplayer
 import abstract, lessonfile
 import soundcard
 
@@ -282,21 +282,24 @@ class Gui(abstract.LessonbasedGui):
                 x.get_children()[0].set_name("")
         ##
         try:
-            n = self.m_t.new_question()
-            if n == Teacher.ERR_PICKY:
-                print "picky!"
+            self.m_t.new_question()
+            # Here, we could check if new_question returned PICKY, but
+            # we choose to ignore it, because it will only happen once
+            # when the user have clicked 'new question', and then set
+            # the picky option in the preferences window and the clicks
+            # 'new question' again. So clicking 'new question' will just
+            # repeat the not answered question.
+            self.g_music_displayer.clear()
+            self.m_t.m_P.play_question()
+            self.g_give_up.set_sensitive(True)
+            self.g_repeat.set_sensitive(True)
+            if self.get_bool('config/picky_on_new_question'):
+                self.g_new.set_sensitive(False)
+            if isinstance(self.m_t.m_P.get_question().music, lessonfile.ChordCommon):
+                self.g_repeat_arpeggio.set_sensitive(True)
             else:
-                self.g_music_displayer.clear()
-                self.m_t.m_P.play_question()
-                self.g_give_up.set_sensitive(True)
-                self.g_repeat.set_sensitive(True)
-                if self.get_bool('config/picky_on_new_question'):
-                    self.g_new.set_sensitive(False)
-                if isinstance(self.m_t.m_P.get_question().music, lessonfile.ChordCommon):
-                    self.g_repeat_arpeggio.set_sensitive(True)
-                else:
-                    self.g_repeat_arpeggio.set_sensitive(False)
-                [btn for btn in self.g_atable.get_children() if isinstance(btn, gtk.Button)][-1].grab_focus()
+                self.g_repeat_arpeggio.set_sensitive(False)
+            [btn for btn in self.g_atable.get_children() if isinstance(btn, gtk.Button)][-1].grab_focus()
         except Exception, e:
             if not self.standard_exception_handler(e, exception_cleanup):
                 raise
