@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os.path
 import inputwidgets, abstract, random, soundcard, gu, mpd
 import gobject
 import runtime
@@ -279,7 +280,7 @@ _("""You have to select some tones practise. Do this on the config page by setti
                 for idx, n in enumerate(('c', 'd', 'e', 'f', 'g', 'a', 'b')):
                     self.set_float('%s_weight' % n, self.m_t.m_P.header.white_keys_weight[idx])
             else:
-                print "white_keys_weight had wrong type"
+                gu.dialog_ok("The white_keys_weight variable in the lesson file '%s' had wrong type" % os.path.abspath(self.m_t.m_P.m_filename), msgtype=gtk.MESSAGE_WARNING)
         else:
             for idx, n in enumerate(('c', 'd', 'e', 'f', 'g', 'a', 'b')):
                 self.set_float('%s_weight' % n, 0.0)
@@ -289,7 +290,7 @@ _("""You have to select some tones practise. Do this on the config page by setti
                 for idx, n in enumerate(('cis', 'dis', 'fis', 'gis', 'ais')):
                     self.set_float('%s_weight' % n, self.m_t.m_P.header.black_keys_weight[idx])
             else:
-                print "black_keys_weight had wrong type"
+                gu.dialog_ok("The black_keys_weight variable in the lesson file '%s' had wrong type" % os.path.abspath(self.m_t.m_P.m_filename), msgtype=gtk.MESSAGE_WARNING)
         else:
             for idx, n in enumerate(('cis', 'dis', 'fis', 'gis', 'ais')):
                 self.set_float('%s_weight' % n, 0.0)
