@@ -15,8 +15,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import errno
-import tempfile
 import textwrap
 import glob
 import xrandom
@@ -113,7 +111,6 @@ keywords = (
     'singinterval',
     'melodicinterval',
     'harmonicinterval',
-    'example',
     'idtone',
     'twelvetone',
     'identifybpm',
@@ -228,6 +225,13 @@ lessonfile_functions = {
         'chordname': (False, lambda s: LabelObject("chordname", s)),
 }
 
+# We need this modified dict to use when parsing data into the
+# sqlite-database. This because we want those data to be in C-locale
+# so that the program can be run in different locales from the same
+# database.
+lessonfile_functions_no_i18n = dict(lessonfile_functions)
+lessonfile_functions_no_i18n['_'] = (False, lambda f: f)
+lessonfile_functions_no_i18n['_i'] = (False, lambda f: f)
 
 def rnc_markup_tokenizer(s):
     """
@@ -1466,7 +1470,16 @@ class LessonFileManager:
         # A list of info about lesson files that where discarded
         # by the lesson file manager
         self.m_discards = []
-        self.parse(debug)
+        self.m_uiddb = {}
+    def old_count(self):
+        """
+        Return the number of different lesson_ids we have in the
+        old statistics directory.
+        """
+        count = 0
+        for p in 'lesson-files', filesystem.user_lessonfiles():
+            count += len(glob.glob("%s/*" % p))
+        return count
     def parse(self, debug):
         lessonpath = ['lesson-files',
                       filesystem.user_lessonfiles()]
@@ -1484,24 +1497,8 @@ class LessonFileManager:
                   or filename.endswith("~") \
                   or not os.path.isfile(filename):
                     continue
-                # We save the returned lesson_id because we need it below
-                lesson_id = self.parse_into_uiddb(filename)
-                if not lesson_id:
-                    continue
-                # We have to check if the lesson file has changed,
-                # and if it has, then the results has to be deleted.
-                # This because Solfege has no way of knowing what kind
-                # of changes has been done to the lesson file.
-                h = hash(open(filename, 'r').read())
-                hash_filename = os.path.join(filesystem.app_data(),
-                           "testresults", "%s_hash" % lesson_id)
-                if os.path.exists(hash_filename):
-                    h2 = int(open(hash_filename, 'r').read())
-                    if h != h2:
-                        dirname = os.path.join(filesystem.app_data(),
-                                "testresults", lesson_id)
-                        for f in os.listdir(dirname):
-                            os.remove(os.path.join(dirname, f))
+                uuid = self.parse_into_uiddb(filename)
+                yield uuid, self.m_uiddb[uuid]
     def parse_into_uiddb(self, filename):
                 """
                 Returns the lesson_id of the parsed file.
@@ -1553,48 +1550,31 @@ class LessonFileManager:
                             [self.m_uiddb[p.header['lesson_id']]]
                     self.m_uiddb[p.header['lesson_id']].append(n)
                 return p.header['lesson_id']
-    def create_lessonfile_index(self):
-        self.m_htmldoc = """<html><head><meta http-equiv="Content-Type" content="text/html; charset=UTF-8"></head><body><p>%s</p>""" % _("This page lists all the lesson files that Solfege can find. You can click on the links to start practising.")
-        d = {}
-        for lesson_id, n in self.m_uiddb.items():
-            if not 'module' in n['header']:
-                print "warning: %s is missing a module declaration" %  self.get(lesson_id, 'filename')
-                continue
-            if n['header']['module'] not in d:
-                d[n['header']['module']] = []
-            d[n['header']['module']].append(lesson_id)
-        for module in d.keys():
-            self.m_htmldoc = "%s<h2>%s</h2>" % (self.m_htmldoc, module)
-            self.m_htmldoc += "<ul>"
-            for n in d[module]:
-                self.m_htmldoc += "<li><a href='solfege:practise/%s'>%s</a>: %s</li>" % (
-                    n,
-                    self.get(n, 'filename'),
-                    self.get(n, 'title'))
-            self.m_htmldoc += "</ul>"
     def get(self, lesson_id, fieldname):
         if fieldname in ('title', 'test', 'module', 'lesson_heading'):
             if fieldname in self.m_uiddb[lesson_id]['header']:
                 return self.m_uiddb[lesson_id]['header'][fieldname]
         if fieldname in self.m_uiddb[lesson_id]:
             return self.m_uiddb[lesson_id][fieldname]
-    def is_test_passed(self, lesson_id):
-        return os.path.exists(os.path.join(filesystem.app_data(),
-            'testresults', lesson_id, 'passed'))
     def ignore_duplicates_with_lesson_id(self, lesson_id):
         """
-        Delete the duplicates with lesson_id, keep the oldest file.
+        Discard the extra entries of lesson_id, keep the oldest file.
         """
         def sort_func(a, b):
             return cmp(a['mtime'], b['mtime'])
         self.m_uiddb[lesson_id].sort(sort_func)
         self.m_uiddb[lesson_id] = self.m_uiddb[lesson_id][0]
-    def delete_not_fn(self, lesson_id, fn):
+    def duplicate_id_keep_this(self, lesson_id, filename):
         """
-        Assumes the lesson_id has duplicate entries.
-        Delete the entries that is not the file fn.
+        For the lesson_id conflicts with lesson_id, keep the lesson_id for
+        the file 'filename', add new lesson_id to the other files.
         """
-        self.m_uiddb[lesson_id] = [d for d in self.m_uiddb[lesson_id] if d['filename'] == fn][0]
+        for info in self.m_uiddb[lesson_id]:
+            if info['filename'] != filename:
+                p = LessonIdParser(info['filename'])
+                p.new_lesson_id()
+                self.parse_into_uiddb(info['filename'])
+        self.m_uiddb[lesson_id] = [d for d in self.m_uiddb[lesson_id] if d['filename'] == filename][0]
     def iterate_duplicated_lesson_id(self):
         """
         Return the a lesson_id if there exist lesson_ids that
@@ -1611,7 +1591,9 @@ class LessonFileManager:
         FIXME: the function name is not very good.
         Return data used to fix things when we have a lesson_id crash.
         """
-        return [{'filename': d['filename'], 'timestr': time.strftime('%c', time.localtime(d['mtime'])), 'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
+        return [{'filename': d['filename'].decode(sys.getfilesystemencoding(), 'replace'),
+            'timestr': time.strftime('%c', time.localtime(d['mtime'])).decode(sys.getdefaultencoding(), 'replace'),
+            'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
 
 def parse_lesson_file_header(filename):
     """
@@ -1625,7 +1607,7 @@ def parse_lesson_file_header(filename):
     r = re.compile("\\header\s*{.*?}", re.MULTILINE|re.DOTALL)
     s = open(filename, 'rU').read()
     m = r.search(s)
-    p = dataparser.Dataparser(predef, lessonfile_functions)
+    p = dataparser.Dataparser(predef, lessonfile_functions_no_i18n)
     p.m_location = os.path.split(filename)
     p.m_ignore_lookup_error = True
     if not m:
