@@ -15,6 +15,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
+import logging.handlers
+import optionparser
+
+opt_parser = optionparser.SolfegeOptionParser()
+options, args = opt_parser.parse_args()
+
+if options.debug:
+    handler = logging.StreamHandler()
+    logging.getLogger().addHandler(handler)
+    logging.getLogger().setLevel(logging.DEBUG)
+else:
+    handler = logging.handlers.MemoryHandler(1)
+    logging.getLogger().addHandler(handler)
+
+
 import webbrowser
 import textwrap
 # We move x-www-browser to the end of the list because on my
@@ -60,9 +76,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import osutils
 import app
 
-opt_parser = app.SolfegeOptionParser()
-options, args = opt_parser.parse_args()
-
 if options.version:
     if buildinfo.is_release():
         rev_info = ""
@@ -90,6 +103,10 @@ if options.warranty:
 runtime.init(options)
 
 import gtk
+# We need to set a uri hook that does nothing, since ubuntu 9.04 rc give
+# GConf errors and GtkWarnings without it.
+gtk.link_button_set_uri_hook(lambda a, b: None)
+
 import utils
 import i18n
 
@@ -152,7 +169,6 @@ import harmonicinterval
 import melodicinterval
 import singinterval
 import nameinterval
-import example
 
 import idbyname
 import dictation
@@ -190,6 +206,189 @@ class MusicViewerWindow(gtk.Dialog):
         fontsize = cfg.get_int('config/feta_font_size=20')
         self.g_music_displayer.display(music, fontsize)
 
+class SelectWinBase(gtk.ScrolledWindow):
+    def __init__(self, app):
+        gtk.ScrolledWindow.__init__(self)
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self._on_focus_in_blocked = False
+        self.m_app = app
+        self.set_size_request(500, 300)
+        self.m_linkbuttons = []
+        self.g_searchentry = None
+    def on_focus_in(self, w, event):
+        """
+        Set the vadjustment so that the window will scroll to make the button
+        that has the focus visible in the scrolled window.
+        """
+        if self._on_focus_in_blocked:
+            return
+        a = w.get_allocation()
+        adj = self.get_vadjustment()
+        if a.y + a.height > adj.value + adj.page_size:
+            if w.get_data('last'):
+                adj.value = adj.upper - adj.page_size
+            else:
+                adj.value = a.y - adj.page_size + a.height
+        elif a.y < adj.value:
+            # If the widget is the first row of the first category, then...
+            if w.get_data('first'):
+                adj.value = 0.0
+            else:
+                adj.value = a.y
+    def on_key_press_event(self, mainwin, event):
+        if event.type == gtk.gdk.KEY_PRESS and event.state == 0:
+            adj = self.get_vadjustment()
+            if event.keyval in (gtk.keysyms.End, gtk.keysyms.KP_End):
+                self.m_linkbuttons[-1].grab_focus()
+                return True
+            elif event.keyval in (gtk.keysyms.Home, gtk.keysyms.KP_Home):
+                if self.g_searchentry:
+                    self.get_vadjustment().value = 0.0
+                    self.g_searchentry.grab_focus()
+                else:
+                    self.m_linkbuttons[0].grab_focus()
+                return True
+            if event.keyval in (gtk.keysyms.Page_Down, gtk.keysyms.KP_Page_Down):
+                # First we find the button that now has the keyboard focus
+                for idx, btn in enumerate(self.m_linkbuttons):
+                    if btn.is_focus():
+                        break
+                # Then we find a button that is approximately adj.page_increment
+                # down and grabs that buttons focus.
+                for to_btn in self.m_linkbuttons[idx:]:
+                    if to_btn.get_allocation().y - adj.value > adj.page_size:
+                        self._on_focus_in_blocked = True
+                        to_btn.grab_focus()
+                        self._on_focus_in_blocked = False
+                        return True
+                self.m_linkbuttons[-1].grab_focus()
+                return True
+            if event.keyval in (gtk.keysyms.Page_Up, gtk.keysyms.KP_Page_Up):
+                for idx, btn in enumerate(self.m_linkbuttons):
+                    if btn.is_focus():
+                        break
+                for to_btn in self.m_linkbuttons:
+                    if to_btn.get_allocation().y > adj.value - adj.page_increment:
+                        self._on_focus_in_blocked = True
+                        to_btn.grab_focus()
+                        self._on_focus_in_blocked = False
+                        return True
+
+
+class FrontPage(SelectWinBase):
+    def __init__(self, app):
+        SelectWinBase.__init__(self, app)
+        main_vbox = gtk.VBox()
+        self.add_with_viewport(main_vbox)
+        hbox = gtk.HBox()
+        hbox.set_border_width(12)
+        main_vbox.pack_start(hbox, False)
+        self.g_searchentry = gtk.Entry()
+        hbox.pack_start(self.g_searchentry, True)
+        self.g_searchentry.connect('activate', self.on_search)
+        gu.bButton(hbox, _("Search"), callback=self.on_search, expand=False)
+        hbox = gu.bHBox(main_vbox)
+        for column in app.m_frontpage_data['columns']:
+            first = True
+            vbox = gtk.VBox()
+            hbox.pack_start(vbox, False)
+            for sect_id, section in enumerate(column):
+                category = gu.hig_dlg_vbox()
+                vbox.pack_start(category, False)
+                b1, b2 = gu.hig_category_vbox(_(section['heading']), spacing=0)
+                category.pack_start(b1, False) 
+                for idx, collection_id in enumerate(section['collection-ids']):
+                    label = gu.ClickableLabel(_(app.m_frontpage_data['collections'][collection_id]['name']))
+                    if first:
+                        label.set_data('first', True)
+                        first = False
+                    label.connect('clicked',
+                          lambda f, collection_id=collection_id:
+                          self.m_app.m_ui.display_exercises(
+                          app.m_frontpage_data['collections'][collection_id]))
+                    self.m_linkbuttons.append(label)
+                    label.connect('focus-in-event', self.on_focus_in)
+                    b2.pack_start(label, True)
+            label.set_data('last', True)
+        self.show_all()
+    def on_search(self, *button):
+        self.m_app.m_ui.display_exercises(
+                {'name': _("Search results for '%s':" % self.g_searchentry.get_text()),
+             'children': [x[0] for x in statistics.db.search(
+                 self.g_searchentry.get_text().lower(), ('uuid',))]})
+
+
+class ExerciseView(SelectWinBase):
+    def __init__(self, app):
+        SelectWinBase.__init__(self, app)
+        self.g_box = None
+        self.show_all()
+    def display_exercises(self, data):
+        """
+        data = {'name': _('Ascending melodic intervals'),
+                 'children': [
+                    '7954749b-307d-43e9-b16f-77a942cfdd86',
+                    'eb76c2c3-414d-4682-8a16-b4d3c5475574',
+                    '3ac10629-fd95-42f6-88f0-ba6e60ecd0b2',]
+                }
+        or a list of dicts like the one above.
+        """
+        if self.g_box:
+            self.g_box.destroy()
+        self.g_box = gu.hig_dlg_vbox()
+        self.add_with_viewport(self.g_box)
+        # The category_list hack here let us avoid an extra level
+        # dict with 'children' in the datafile for Exercise views that display
+        # only one category of exercises.
+        if not data['children']:
+            self.g_box.pack_start(gtk.Label(_("No matches")))
+            self.show_all()
+            return
+        if isinstance(data['children'][0], basestring):
+            category_list = [data]
+        else:
+            category_list = data['children']
+        first = True
+        self.m_linkbuttons = []
+        for category in category_list:
+            b1, b2 = gu.hig_category_vbox(_(category['name']), spacing=0)
+            self.g_box.pack_start(b1, False)
+            for idx, uuid in enumerate(category['children']):
+                txt = _(statistics.db.get_field(uuid, "title"))
+                max_chars = 35
+                if len(txt) > max_chars:
+                    # translators, this is used when the gui feel the string
+                    # is too long to be displayed. On many languages it is
+                    # logical to add the dots '...' after the truncated word.
+                    # Anyway, as always with messages with a | inside, dont
+                    # include or translate "truncated|" in this string. Just
+                    # enter the 3 dots, or whatever is correct for your language.
+                    txt2 = _i("truncated|%s...") % txt[:max_chars]
+                else:
+                    txt2 = txt
+                label = gu.ClickableLabel(txt2)
+                if txt != txt2:
+                    label.set_tooltip_text(txt)
+                self.m_linkbuttons.append(label)
+                if first:
+                    label.set_data('first', True)
+                    first = False
+                label.connect('clicked', self.on_link_clicked, uuid)
+                label.connect('focus-in-event', self.on_focus_in)
+                b2.pack_start(label, False)
+        label.set_data('last', True)
+        self.show_all()
+        self.m_linkbuttons[0].grab_focus()
+    def on_link_clicked(self, w, uuid):
+        self.m_app.practise_lesson_id(uuid)
+    def on_end_practise(self, w=None):
+        pass
+
+
+class TestsView(ExerciseView):
+    def on_link_clicked(self, w, uuid):
+        self.m_app.test_lesson_id(uuid)
+
 
 class MainWin(gtk.Window, cfg.ConfigUtils):
     default_learning_tree = ('solfege', 'learningtree.txt')
@@ -231,7 +430,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.main_box = gtk.VBox()
         self.main_box.show()
         self._vbox.pack_start(self.main_box)
-        self.display_docfile_in_mainwin('welcome.html')
+        self.display_frontpage()
     def change_learning_tree(self, tree_fn):
         """
         Change to a different learning tree.
@@ -265,7 +464,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             gu.dialog_ok(_("Learning tree '%s' not found. Using default tree." % filename))
             if splash_win:
                 splash_win.show()
-        self.create_practise_and_test_menu()
     def setup_menu(self):
         self.m_action_groups['Exit'].add_actions([
           ('FileMenu', None, _('_File')),
@@ -278,7 +476,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
           ('EditMenu', None, _("_Edit")),
           ('TreeEditor', None, _('Edit learning tree'), None, None,
             self.do_tree_editor),
-          ('TestsMenu', None, _("_Tests")),
           ('ExportTrainingSet', None, _('New training set editor window'), None, None,
             self.new_training_set_editor),
           ('EditPractiseSheet', None, _('Ear training test printout editor'), None, None,
@@ -286,6 +483,10 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
           ('OpenPreferencesWindow', 'gtk-preferences', None, '<ctrl>F12', None,
             self.open_preferences_window),
           ('HelpMenu', None, _('_Help')),
+          ('FrontPage', None, _('_Front page'), 'F5', None,
+              self.display_frontpage),
+          ('TestsPage', None, _('_Tests page'), 'F6', None,
+              self.display_testpage),
           ('HelpHelp', 'gtk-help', _('_Help on the current exercise'), 'F1', None,
             lambda o: self.m_app.please_help_me()),
           ('HelpTheory', None, _('_Music theory on the current exercise'), 'F3', None, lambda o: self.m_app.show_exercise_theory()),
@@ -359,77 +560,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         s += "</menu></menu></menubar>"
         self.m_action_groups['NotExit'].add_actions(actions)
         self.m_learning_trees_merge_id = self.g_ui_manager.add_ui_from_string(s)
-    def create_practise_and_test_menu(self):
-        """
-        Create the Practise menu.
-        This function has to be called after we have cleared out any
-        lesson_id crashes.
-        """
-        def escape(s):
-            return s.replace('&','&amp;').replace("'", "&apos;").replace('"', '&quot;')
-        actions = []
-        if self.m_practise_and_test_merge_id:
-            self.g_ui_manager.remove_ui(self.m_practise_and_test_merge_id)
-            self.g_ui_manager.ensure_update()
-        # This method can be called more than once. So we must first
-        # remove any actions from the last time it was called.
-        for action in self.m_action_groups['Exercises'].list_actions():
-            self.m_action_groups['Exercises'].remove_action(action)
-        # We use this counter to make sure all Action names are unique.
-        # This to avoid warnings if a toplevel menu and a submenu has
-        # the same name
-        id_counter = 0
-        s = "<menubar name='Menubar'><placeholder name='ExercisesPlaceHolder'>"
-        tests = "<menu action='TestsMenu'>"
-        for menu in self.m_tree.m_menus:
-            id_counter += 1
-            self.m_action_groups['Exercises'].add_action(
-                gtk.Action("%s%i" % (menu.name, id_counter),
-                           _(menu.name), None, None))
-            s += "<menu action='%s%i'>" % (escape(menu.name), id_counter)
-            # The following line might look strange here, but it makes sure
-            # that the menus on the menu bar stay at the correct order after
-            # we save the learning tree in the learning tree editor.
-            self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='%s%i'/></menubar>" % (escape(menu.name), id_counter))
-            for topic in menu.children:
-                id_counter += 1
-                s += "<menu action='%s%i'>" % (escape(topic['name']), id_counter)
-                tests += "<menu action='TEST_%s%i'>" % (escape(topic['name']), id_counter)
-                actions.append(("%s%i" % (topic['name'], id_counter),
-                                None, _(topic['name'])))
-                actions.append(('TEST_%s%i' % (topic['name'], id_counter),
-                               None, _(topic['name'])))
-                for lesson_id in topic.children:
-                    if self.m_tree.m_visibilities[lesson_id] > self.m_tree.m_visibility:
-                        continue
-                    menutitle = self.m_app.lessonfile_manager.m_uiddb[lesson_id]['header']['title']
-                    id_counter += 1
-                    actions.append((
-                        "%s%i" % (lesson_id, id_counter),
-                        None, menutitle, None, None,
-                        lambda o, e=lesson_id: self.m_app.practise_lesson_id(e)))
-                    if statistics.db.is_test_passed(lesson_id):
-                        menutitle = _("%s (passed)") % menutitle
-                    if self.m_app.lessonfile_manager.get(lesson_id, 'test'):
-                        actions.append((
-                            'TEST_%s%s' % (lesson_id, id_counter), None,
-                            menutitle,
-                            None, None,
-                            lambda o, e=lesson_id: self.m_app.test_lesson_id(e)))
-                        tests += "<menuitem action='TEST_%s%i' />" % (escape(lesson_id), id_counter)
-                    s += "<menuitem action='%s%i' />" % (escape(lesson_id), id_counter)
-                s += "</menu>"
-                tests += "</menu>"
-            s += "</menu>"
-        self.m_action_groups['Exercises'].add_actions(actions)
-        tests += "</menu>"
-        s += tests
-        s += "</placeholder></menubar>"
-        # The following line might look strange here, but it makes sure
-        # that the menus on the menu bar stay at the correct order after
-        # we save the learning tree in the learning tree editor.
-        self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='TestsMenu'/></menubar>")
-        self.m_practise_and_test_merge_id = self.g_ui_manager.add_ui_from_string(s)
     def show_help_on_current(self):
         """
         Show the menu entries for the exercise help and music theory
@@ -608,12 +738,9 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         win.show()
     def post_constructor(self):
         global splash_win
-        self.m_practise_and_test_merge_id = None
         self.m_learning_trees_merge_id = None
-        self.on_learning_tree_changed()
         self.create_learning_trees_menu()
         self.g_ui_manager.add_ui_from_file("help-menu.xml")
-        self.box_dict['docviewer'].grab_focus()
         if self.m_app.m_sound_init_exception is not None:
             if splash_win:
                 splash_win.destroy()
@@ -773,6 +900,56 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             self.g_help_browser.show()
             self.g_help_browser.present()
             self.g_help_browser.show_docfile(fn, anchor)
+    def display_testpage(self, w=None):
+        if self.m_viewer != 'testspage':
+            if self.m_viewer:
+                self.box_dict[self.m_viewer].hide()
+        if 'testspage' not in self.box_dict:
+            frontpage = TestsView(self.m_app)
+            self.box_dict['testspage'] = frontpage
+            self.main_box.pack_start(frontpage)
+        self.m_viewer = 'testspage'
+        self.box_dict[self.m_viewer].show()
+        self.box_dict[self.m_viewer].get_vadjustment().value = 0.0
+        v  = []
+        for k in ('asc-mel-int', 'des-mel-int', 'mel-int', 'harm-int',
+                'sing-int', 'chords', 'cadences', 'solfa-syll'):
+            # some 'collections' in the datafile contains two or more records
+            collection = self.m_app.m_frontpage_data['collections'][k]
+            if type(collection['children'][0]) == dict:
+                for kk in collection['children']:
+                    v.append(kk)
+                continue
+            v.append(collection)
+        v = {'name': _("Tests"),
+            'children': v}
+        self.box_dict[self.m_viewer].display_exercises(v)
+    def display_frontpage(self, widget=None):
+        if self.m_viewer != 'frontpage':
+            if self.m_viewer:
+                self.box_dict[self.m_viewer].hide()
+        if 'frontpage' not in self.box_dict:
+            frontpage = FrontPage(self.m_app)
+            self.box_dict['frontpage'] = frontpage
+            self.main_box.pack_start(frontpage)
+            w, h = frontpage.get_children()[0].get_children()[0].size_request()
+            if w > 500:
+                frontpage.set_size_request(w + 24, 300)
+        self.m_viewer = 'frontpage'
+        self.box_dict[self.m_viewer].show()
+        self.box_dict[self.m_viewer].g_searchentry.grab_focus()
+        self.box_dict[self.m_viewer].get_vadjustment().value = 0.0
+    def display_exercises(self, data):
+        if self.m_viewer != 'exerciseviewer':
+            if self.m_viewer:
+                self.box_dict[self.m_viewer].hide()
+        if 'exerciseviewer' not in self.box_dict:
+            eview = ExerciseView(self.m_app)
+            self.box_dict['exerciseviewer'] = eview
+            self.main_box.pack_start(eview)
+        self.m_viewer = 'exerciseviewer'
+        self.box_dict[self.m_viewer].show()
+        self.box_dict[self.m_viewer].display_exercises(data)
     def display_docfile_in_mainwin(self, fn, anchor=None):
         self._show_docviewer()
         self.box_dict['docviewer'].read_docfile(fn, anchor)
@@ -835,15 +1012,14 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.box_dict[self.m_viewer].g_notebook.get_nth_page(0).show()
         self.box_dict[self.m_viewer].g_notebook.set_current_page(0)
         self.box_dict[self.m_viewer].exit_test_mode()
-        # rebuild the menus in case we hav passed a test.
-        self.create_practise_and_test_menu()
     def new_training_set_editor(self, widget):
         dlg = TrainingSetDialog(self.m_app)
         dlg.show_all()
     def new_practisesheet_editor(self, widget):
         dlg = PractiseSheetDialog(self.m_app)
         dlg.show_all()
-
+    def handle_lesson_id_crash(self):
+        lessonfilegui.handle_lesson_id_crash(lessonfile.manager)
 
 import locale
 locale.setlocale(locale.LC_NUMERIC, "C")
@@ -858,15 +1034,12 @@ if splash_win:
     splash_win.show_progress("creating LessonfileManager")
 lessonfile_manager = lessonfile.LessonFileManager(options.debug)
 lessonfile.manager = lessonfile_manager
-statistics.db = statistics.DB()
-
 if splash_win:
-    splash_win.hide()
-lessonfilegui.handle_lesson_id_crash(lessonfile_manager)
-if splash_win:
-    splash_win.show()
-
-lessonfile_manager.create_lessonfile_index()
+    if not os.path.exists(statistics.DB.get_statistics_filename()):
+        splash_win.show_progress("Importing old statistics:")
+def f(s):
+    splash_win.show_progress("Importing old statistics: %i%%" % s)
+statistics.db = statistics.DB(f)
 
 if splash_win:
     splash_win.show_progress("creating MainWin")
@@ -896,5 +1069,6 @@ def start_app(datadir):
             sys.__excepthook__(t, msg, traceback)
     if not options.disable_exception_handler:
         sys.excepthook = ef
+    print time.time() - start_time
     gtk.main()
 
