@@ -118,7 +118,6 @@ class Gui(abstract.IntervalGui):
             # automatically when we change lesson file.
             if self.m_t.m_custom_mode:
                 self.on_end_practise()
-                #self.on_start_practise()
         for i in range(self.get_int('maximum_number_of_intervals')):
             self.add_watch('ask_for_intervals_%i' % i, _f)
         self.add_watch('number_of_intervals', _f)
@@ -150,10 +149,7 @@ class Gui(abstract.IntervalGui):
             if not self.m_t.m_tonika:
                 return
             i = mpd.Interval()
-            if (not midi_int # buttons interface does not set midi_int. And
-                             # since it assumes all intervals go up, we have
-                             # to check and set the direction.
-                and self.m_t.m_question[0] < 0):
+            if self.m_t.m_question[0] < 0:
                 i.set_from_int(-interval)
             else:
                 i.set_from_int(interval)
@@ -163,7 +159,7 @@ class Gui(abstract.IntervalGui):
             track.note(4, n.semitone_pitch())
             soundcard.synth.play_track(track)
             return
-        if mouse_button not in (1, self.keyboard_accel):
+        if mouse_button != 1:
             return
         if self.m_t.m_app.m_test_mode and self.m_t.q_status == self.QSTATUS_NO:
             self.g_flashbar.flash(_("Click 'Start test' to begin."))
@@ -183,20 +179,15 @@ class Gui(abstract.IntervalGui):
             self.g_flashbar.flash(_("You have already identified this interval"))
             return
         if not (-17 < interval < 17):
-            self.g_flashbar.flash(_("Ignoring intervals greater than major tenth."))
+            self.g_flashbar.flash(_("Ignoring intervals greater than decim."))
             self.g_input.forget_last_tone()
             return
         self.m_answer.append(interval)
         d = self.m_t.m_question[len(self.m_answer)-1]
-        md = d / abs(d)
+        d = d / abs(d)
+        md = d
         if self.g_input.know_directions():
             md = 1
-        # If the user answers with keyboard shortcuts, then their answer does
-        # not say if the interval goes up or down. So we assumes the user
-        # got the direction correct.
-        if mouse_button == self.keyboard_accel:
-            if self.m_answer[-1] > 0 and d < 0:
-                self.m_answer[-1] = -self.m_answer[-1]
         if not self.msg:
             self.msg = utils.int_to_intervalname(interval * md, 1, 1) + "+ ..."
         else:
