@@ -1,3 +1,19 @@
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import optparse
 import sys
 
@@ -7,7 +23,7 @@ class SolfegeOptionParser(optparse.OptionParser):
         self.add_option('-v', '--version', action='store_true', dest='version')
         self.add_option('-w', '--warranty', action='store_true', dest='warranty',
             help=_('Show warranty and copyright.'))
-        self.add_option('--no-splash', action='store_false', dest='no_splash',
+        self.add_option('--no-splash', action='store_true', dest='no_splash',
             help=_('Do not show the startup window.'),
             default=False)
         self.add_option('--verbose-sound-init', action='store_true',
