@@ -16,6 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+import logging
 import sqlite3
 import hashlib
 import pickle
@@ -46,12 +47,31 @@ def hash_of_lessonfile(filename):
 
 
 class DB(object):
-    def __init__(self):
+    """
+    This DB class is currently located in src.statistics.py, but it might
+    move to a separate/another file later, since it will store more data
+    than just statistics.
+   
+    At startup on a fresh install:
+    * Scan all lesson files (only standard files, not regression) to
+      get uuid, filename, title, hash etc into the lessonfiles table.
+
+    On every startup:
+    * After the GUI is ready, we scan all lesson files in the background
+      to check if any files have changed.
+
+    When we start to practise an exercise:
+    * Check if the lesson file has changed, and if so, we delete the statistics.
+    """
+    def __init__(self, callback=None):
+        """
+        callback is called to display progress when scanning lesson files.
+        """
         try:
             if testsuite_is_running:
                 statistics_filename = ":memory:"
         except NameError:
-            statistics_filename = os.path.join(filesystem.app_data(), "statistics.sqlite")
+            statistics_filename = self.get_statistics_filename()
         self.must_recreate = not os.path.exists(statistics_filename)
         try:
             if testsuite_is_running:
@@ -61,13 +81,22 @@ class DB(object):
         self.conn = sqlite3.connect(statistics_filename)
         self.setup_tables()
         if self.must_recreate:
+            self.scan_lessonfiles(callback)
             self.read_old_data()
+    @staticmethod
+    def get_statistics_filename():
+        return os.path.join(filesystem.app_data(), "statistics.sqlite")
     def setup_tables(self):
         """
-        lessonfiles table. Columnns
+        lessonfiles table.
+        ==================
         uuid: the uuid of the lesson file, as written in the lessonfile header.
         hash: hash value of the file the last time some statistics for the
               file was entered into the database.
+        test_result: The result of the last time a test was made.
+        test_passed: If the test was passed the last time taken.
+        title: The lesson file, from the lesson file header
+        module: The exercise module, taken from the lessonfile header
         """
         self.conn.execute('''create table if not exists lessonfiles
             (uuid text primary key not null,
@@ -75,41 +104,99 @@ class DB(object):
              test_result float default 0.0,
              test_passed int default 0
             )''')
+        columns = [x[1] for x in
+              self.conn.execute('pragma table_info(lessonfiles)').fetchall()]
+        if 'title' not in columns:
+            self.conn.execute('alter table lessonfiles add column title txt')
+        if 'module' not in columns:
+            self.conn.execute('alter table lessonfiles add column module txt')
+        if 'filename' not in columns:
+            self.conn.execute('alter table lessonfiles add column filename txt')
         self.conn.execute('''create table if not exists sessions
             (uuid text, timestamp int, answerkey text, guessed text, count int, unique (uuid, timestamp, answerkey, guessed) )''')
         self.conn.execute('''create table if not exists tests
             (uuid text, timestamp int, answerkey text, guessed text, count int, unique (uuid, timestamp, answerkey, guessed) )''')
+    def get_field(self, uuid, fields):
+        if not isinstance(fields, (tuple, list)):
+            vfields = [fields]
+        else:
+            vfields = fields
+        row = self.conn.execute('select %s from lessonfiles where uuid=?'
+                                % ', '.join(vfields), (uuid,)).fetchone()
+        if isinstance(fields, (tuple, list)):
+            return row
+        else:
+            return row[0]
+    def scan_lessonfiles(self, callback=None):
+        """
+        Read info about standard lesson files into the lessonfiles table.
+        Current lessonfile manager code cause us to parse the user contributed
+        files in ~/lessonfiles too.
+        """
+        count = 0
+        cc = lessonfile.manager.old_count()
+        old_uuid = None
+        uuid_count = 0
+        progress = 0
+        old_progress = 0
+        for uuid, data in lessonfile.manager.parse(False):
+            if callback and uuid != old_uuid:
+                uuid_count += 1
+                old_uuid = uuid
+            self.set_lessonfile_data(uuid, data['header']['title'],
+                    data['header']['module'], data['filename'])
+            if callback:
+                progress = uuid_count * 100 / cc
+                if old_progress != progress:
+                    callback(progress)
+                    old_progress = progress
+            count += 1
+        logging.info("scan_lessonfiles: scanned %i files." % count)
+    def set_lessonfile_data(self, uuid, title, module, filename):
+        """
+        Set the values title, module, filename and hash of uuid
+        if the lesson file has changed.
+        """
+        row = self.conn.execute("select uuid, hash from lessonfiles where uuid=?",
+                (uuid,)).fetchone()
+        hashvalue = hash_of_lessonfile(filename)
+        if row:
+            if hashvalue != row[1]:
+                self.validate_stored_statistics(uuid, filename)
+                self.conn.execute("update lessonfiles "
+                    "set title=:title, module=:module, filename=:filename, "
+                    "hash=:hashvalue "
+                    "where uuid=:uuid", locals())
+        else:
+            self.conn.execute("insert into lessonfiles "
+                    "(uuid, title, module, filename, hash) "
+                    "values (?, ?, ?, ?, ?)",
+                    (uuid, title, module, filename, hashvalue))
+        self.conn.commit()
     def read_old_data(self):
         """
         This function should be run once to import the old format statistics
         from ~/.solfege/statistics and into the sqlite database.
+        We will import statistics for all uuids that exist in the lessonfiles table.
         """
         st_home = os.path.join(filesystem.app_data(), 'statistics')
-        for fn in os.listdir(st_home):
-            if (not fn.endswith('_hash')) and (fn != 'statistics_version'):
-                hashvalue_filename = os.path.join(st_home, fn + "_hash")
-                if not os.path.exists(hashvalue_filename):
-                    # The ~/.solfege/statistics directory contained directories
-                    # namad by the uuid, and files uuid_hash that contained the hash
-                    # value. The old statistics code would delete the uuid_hash file
-                    # and the statistics stored in the directory if the lesson file
-                    # changed, but not delete the directory itself. So there might be
-                    # empty directories lying around.
-                    continue
-                hashvalue = hash_of_lessonfile(lessonfile.manager.get(fn, 'filename'))
-                self.conn.execute("insert into lessonfiles (uuid, hash) values(?, ?)",
-                                  (fn, hashvalue))
-        self.conn.commit()
-        c = self.conn.execute('select uuid from lessonfiles')
-        for uuid in c:
-            for timestamp in os.listdir(os.path.join(st_home, uuid[0])):
-                f = open(os.path.join(st_home, uuid[0], timestamp), 'r')
+        ign = 0
+        imp = 0
+        for row in self.conn.execute('select uuid from lessonfiles'):
+            uuid = row[0]
+            if not os.path.exists(os.path.join(st_home, uuid)):
+                ign += 1
+                continue
+            for timestamp in os.listdir(os.path.join(st_home, uuid)):
+                f = open(os.path.join(st_home, uuid, timestamp), 'r')
                 session = pickle.load(f)
                 f.close()
                 for correct in session.keys():
                     for guess in session[correct]:
                         self.conn.execute("insert into sessions values(?, ?, ?, ?, ?)",
-                        (uuid[0], timestamp, unicode(correct), unicode(guess), session[correct][guess]))
+                        (uuid, timestamp, unicode(correct), unicode(guess), session[correct][guess]))
+                imp += 1
+        logging.info("imported statistics for %i lessonfiles, ignored %i" % (imp, ign))
         self.conn.commit()
     def last_test_result(self, uuid):
         """
@@ -132,6 +219,42 @@ class DB(object):
         if row:
             return row[0]
         return False
+    def validate_stored_statistics(self, uuid, filename):
+        """
+        Check if the lesson file has changed since last time statistics for
+        this lessonfile was added. Remove old statistics if the 
+        m_t.m_P has already changed to the new file when this method is called.
+        """
+        cursor = self.conn.cursor()
+        row = cursor.execute("select hash from lessonfiles where uuid=?", (uuid,)).fetchone()
+        cur_lessonfile_hash_value = hash_of_lessonfile(filename)
+        if not row:
+            # Ususally the uuid exists in the database, but when running the
+            # test suite, it does not, so we have the code here to add it.
+            cursor.execute("insert into lessonfiles (uuid, hash) values(?, ?)",
+                    (uuid, cur_lessonfile_hash_value))
+            self.conn.commit()
+        else:
+            # row[0] is the hash value of the lesson file when all the existing
+            # statistics for the lesson file with this uuid was saved.
+            if row[0] != cur_lessonfile_hash_value:
+                cursor.execute("delete from sessions where uuid=?", (uuid,))
+                cursor.execute("update lessonfiles "
+                    "set hash=?, test_passed=0, test_result=0.0 where uuid=?",
+                    (cur_lessonfile_hash_value, uuid))
+                self.conn.commit()
+    def search(self, searchfor, columns):
+        """
+        Search for searchfor in the title column (and possible other places
+        later). Return the columns named by the list columns.
+        """
+        searchfor = "%%%s%%" % searchfor
+        n =  self.conn.execute("select %s " % ", ".join(columns)
+             + "from lessonfiles where title like ?", (searchfor,)).fetchall()
+        if not n:
+            return []
+        return n
+
 
 class AbstractStatistics(object):
     def __init__(self, teacher):
@@ -157,33 +280,17 @@ class AbstractStatistics(object):
         answers the user have given and the values are the number of times
         that particular answer has been given.
         """
-        q = db.conn.execute("select answerkey, guessed, sum(count) from sessions where uuid=? and timestamp>? group by answerkey, guessed", (self.m_t.m_P.header.lesson_id, 0))
+        if seconds == -1:
+            q = db.conn.execute("select answerkey, guessed, sum(count) from sessions where uuid=? group by answerkey, guessed", (self.m_t.m_P.header.lesson_id,))
+        elif seconds == 0:
+            q = db.conn.execute("select answerkey, guessed, sum(count) from sessions where uuid=? and timestamp=? group by answerkey, guessed", (self.m_t.m_P.header.lesson_id, self.m_timestamp))
+        else:
+            q = db.conn.execute("select answerkey, guessed, sum(count) from sessions where uuid=? and timestamp>? group by answerkey, guessed", (self.m_t.m_P.header.lesson_id, self.m_timestamp - seconds))
         ret = {}
         for answer, guess, count in q.fetchall():
             ret.setdefault(answer, {})
             ret[answer][guess] = count
         return ret
-    def lessonfile_changed(self):
-        """
-        Check if the lesson file has changed since last time statistics for
-        this lessonfile was added. Remove old statistics if the 
-        m_t.m_P has already changed to the new file when this method is called.
-        """
-        cursor = db.conn.cursor()
-        row = cursor.execute("select hash from lessonfiles where uuid=?", (self.m_t.m_P.header.lesson_id,)).fetchone()
-        cur_lessonfile_hash_value = hash_of_lessonfile(self.m_t.m_P.m_filename)
-        if not row:
-            cursor.execute("insert into lessonfiles (uuid, hash) values(?, ?)",
-                    (self.m_t.m_P.header.lesson_id, cur_lessonfile_hash_value))
-        else:
-            assert row
-            # row[0] is the hash value of the lesson file when all the existing
-            # statistics for the lesson file with this uuid was saved.
-            if row[0] != cur_lessonfile_hash_value:
-                ret = cursor.execute("delete from sessions where uuid=?", (self.m_t.m_P.header.lesson_id,))
-                ret = cursor.execute("update lessonfiles set hash=?, test_passed=0, test_result=0.0 where uuid=?",
-                    (cur_lessonfile_hash_value, self.m_t.m_P.header.lesson_id))
-                db.conn.commit()
     def reset_session(self):
         """
         Start a new practise session.
@@ -193,29 +300,23 @@ class AbstractStatistics(object):
         self.m_test_mode = True
     def exit_test_mode(self):
         self.m_test_mode = False
-        row = db.conn.execute("select * from lessonfiles where uuid=?",
-                (self.m_t.m_P.header.lesson_id,)).fetchone()
         test_result = db.last_test_result(self.m_t.m_P.header.lesson_id)
-        passed = test_result >= self.m_t.m_P.get_test_requirement()
-        if row:
-            db.conn.execute("update lessonfiles "
-                          "set test_result=?, test_passed=? where uuid=?",
-                          (test_result, passed, self.m_t.m_P.header.lesson_id,))
-        else:
-            db.conn.execute("insert into lessonfiles  "
-                            "(uuid, hashvalue, test_result, test_passed) "
-                            "values(?, ?, ?, ?) ",
-                   (self.m_t.m_P.header.lesson_id,
-                    hash_of_lessonfile(lessonfile.manager.get(fn, 'filename')),
-                    test_result,
-                    passed
-                    ))
+        db.conn.execute("update lessonfiles "
+                        "set test_result=?, test_passed=? where uuid=?",
+            (test_result,
+             test_result >= self.m_t.m_P.get_test_requirement(),
+             self.m_t.m_P.header.lesson_id,))
         db.conn.commit()
     def _add(self, question, answer):
         """
         Register that for the question 'question' the user answered 'answer'.
         """
         assert self.m_timestamp
+        # tuples must be converted to str to store them in sqlite.
+        if isinstance(question, tuple):
+            question = str(question)
+        if isinstance(answer, tuple):
+            answer = str(answer)
         table = {True: 'tests', False: 'sessions'}[self.m_test_mode]
         cursor = db.conn.cursor()
         row = cursor.execute(
@@ -242,11 +343,6 @@ class AbstractStatistics(object):
         self._add(question, answer)
     def add_correct(self, answer):
         self._add(answer, answer)
-    def key_to_pretty_name(self, key):
-        for question in self.m_t.m_P.m_questions:
-            if question.name.cval == key:
-                return question.name
-        return key
     def get_last_test_result(self):
         """
         Return the test result of the last test ran for this lesson file.
@@ -307,10 +403,16 @@ class AbstractStatistics(object):
 
 class LessonStatistics(AbstractStatistics):
     def key_to_pretty_name(self, key):
+        def ff(x):
+            t = eval(x)
+            if type(t) == tuple:
+                return lessonfile.LabelObject(t[0], t[1])
+            else:
+                return t
         for question in self.m_t.m_P.m_questions:
             if question.name.cval == key:
-                return question.name
-        return key
+                return ff(question.name)
+        return ff(key)
 
 class IntervalStatistics(AbstractStatistics):
     def key_to_pretty_name(self, key):
