@@ -31,7 +31,7 @@ class MatrixTable(gtk.VBox):
         hbox.pack_start(frame, False)
         t = gtk.Table()
         frame.add(t)
-        st_data = st.get_statistics(0)
+        st_data = st.get_statistics(seconds)
         keys = st.get_keys(True)
         for x in range(len(keys)):
             t.attach(gtk.VSeparator(), x*2+1, x*2+2, 0, len(keys)*2)
@@ -88,12 +88,7 @@ class PercentagesTable(gtk.Frame):
         for box in self.boxdict.values():
             for o in box.get_children():
                 o.destroy()
-        for k in statistics.get_keys():
-            # FIXME  Has this bug been fixed? This should NEVER happen.
-            if k is None:
-                print "WARNING: statistics: key==None. Please, please report to"
-                print "bug-solfege@gnu.org if you get this message. Preferably,"
-                print "make a copy ~/.solfege so I can debug this."
+        for k in statistics.get_keys(True):
             l = lessonfilegui.new_labelobject(statistics.key_to_pretty_name(k))
             l.set_alignment(0.0, 0.5)
             self.boxdict['keys'].pack_start(l)
