@@ -5,9 +5,9 @@
 
 import unittest
 from src import statistics
-statistics.db = statistics.DB()
 from src import app
 from src import lessonfile
+from src import optionparser
 from src import idbyname
 
 class TestDB(unittest.TestCase):
@@ -20,10 +20,9 @@ class TestDB(unittest.TestCase):
         self.assertEqual(statistics.hash_lessonfile_text(s1),
                          statistics.hash_lessonfile_text(s3))
     def test_two(self):
-        opt_parser = app.SolfegeOptionParser()
+        opt_parser = optionparser.SolfegeOptionParser()
         options, args = opt_parser.parse_args()
-        lessonfile.manager = lessonfile.LessonFileManager(options.debug)
-        a = app.SolfegeApp(options, None, None)
+        a = app.SolfegeApp(options, None, lessonfile.manager)
         t = idbyname.Teacher('idbyname', a)
         t.set_lessonfile('lesson-files/chord-min-major')
         t.m_statistics.reset_session()
