@@ -24,12 +24,14 @@ import buildinfo
 from dataparser import Dataparser
 import lessonfile
 import osutils
+import statistics
+import lessonfilegui
 
 
 # The definition of a training set will be stored in a file in the
 # ~/.solfege/trainingsets
 
-class TrainingSetDialog(gu.EditorDialogBase):
+class TrainingSetDialog(gu.EditorDialogBase, lessonfilegui.ExercisesMenuAddIn):
     fileformat_version = 1
     STORE_LESSON_ID = 0
     STORE_TITLE = 1
@@ -128,20 +130,23 @@ class TrainingSetDialog(gu.EditorDialogBase):
            else:
                self.g_ui_manager.get_widget("/ExportToolbar/Remove").set_sensitive(False)
     def on_add_lesson_clicked(self, button):
-        menu = self.create_learning_tree_menu()
-        menu.popup(None, None, None, 1, 0)
+        try:
+            self.menu
+        except AttributeError:
+            self.menu = self.create_learning_tree_menu()
+        self.menu.popup(None, None, None, 1, 0)
     def on_select_exercise(self, item):
         """
         This method is called when the user has selected an exercise to
         add.
         """
         lesson_id = item.get_data('lesson_id')
-        module = self.m_app.lessonfile_manager.get(lesson_id, 'module')
+        module = statistics.db.get_field(lesson_id, 'module')
         if module not in ('harmonicinterval', 'melodicinterval', 'idbyname'):
             print "Only harmonicinterval, melodicinterval and idbyname module exercises are working now. Ignoring..."
             return
         if module == 'idbyname':
-            fn = self.m_app.lessonfile_manager.get(lesson_id, 'filename')
+            fn = statistics.db.get_field(lesson_id, 'filename')
             p = lessonfile.LessonfileCommon()
             p.parse_file(fn)
             if not [q for q in p.m_questions if isinstance(q.music, lessonfile.MpdParsable)]:
@@ -155,76 +160,7 @@ class TrainingSetDialog(gu.EditorDialogBase):
         """
         Return a string we use the name the lesson file in the GUI.
         """
-        return self.m_app.lessonfile_manager.get(lesson_id, 'title') \
-            + " (%s)" % self.m_app.lessonfile_manager.get(lesson_id, 'filename')
-    def create_learning_tree_menu(self):
-        """
-        Create and return a gtk.Menu object that has submenus that
-        let us select all lessons on the learning tree.
-        """
-        def create_submenu(menudata, parent_name):
-            """
-            Menudata is a dict. Key we will use:
-            'name'
-            'children': list of dict or string (or maybe both?).
-            """
-            if isinstance(menudata, list):
-                menu = gtk.Menu()
-                for lesson_id in menudata:
-                    if self.m_app.lessonfile_manager.get(lesson_id, 'module') not in ('melodicinterval', 'harmonicinterval', 'idbyname'):
-                        continue
-                    # We don't want to add these lesson files because we know
-                    # that they cannot be exported. It would be better
-                    # to catch these with a more generit algorithm, but
-                    # then we would have to parse all the files, and that
-                    # would be too slow.
-                    if lesson_id in (
-                            # melodic-interval-self-config
-                            "f62929dc-7122-4173-aad1-4d4eef8779af",
-                            # harmonic-interval-self-config
-                            "466409e7-9086-4623-aff0-7c27f7dfd13b",
-                            # the csound-fifth-* files:
-                            "b465c807-d7bf-4e3a-a6da-54c78d5b59a1",
-                            "aa5c3b18-664b-4e3d-b42d-2f06582f4135",
-                            "5098fb96-c362-45b9-bbb3-703db149a079",
-                            "3b1f57e8-2983-4a74-96da-468aa5414e5e",
-                            "a06b5531-7422-4ea3-8711-ec57e2a4ce22",
-                            "e67c5bd2-a275-4d9a-96a8-52e43a1e8987",
-                            "1cadef8c-859e-4482-a6c4-31bd715b4787",
-                            ):
-                        continue
-                    i = gtk.MenuItem(
-                        self.m_app.lessonfile_manager.get(lesson_id, 'title'))
-                    i.set_data('lesson_id', lesson_id)
-                    i.set_data('menus', parent_name)
-                    i.connect('activate', self.on_select_exercise)
-                    menu.append(i)
-                return menu
-            item = gtk.MenuItem(_(menudata['name']))
-            menu = gtk.Menu()
-            for m in menudata['children']:
-                i = gtk.MenuItem(_(m['name']))
-                menu.append(i)
-                i.set_submenu(create_submenu(m['children'], parent_name + [_(m['name'])]))
-            item.set_submenu(menu)
-            return item
-        menu = gtk.Menu()
-        for m in self.m_app.m_ui.m_tree.m_menus:
-            menu.append(create_submenu(m, [_(m['name'])]))
-
-        menu.show_all()
-        self._menu_hide_stuff(menu)
-        return menu
-    def _menu_hide_stuff(self, menu):
-        """
-        Hide the menu if it has no menu items, or all menu items are hidden.
-        """
-        for sub in menu.get_children():
-            assert isinstance(sub, gtk.MenuItem)
-            if sub.get_submenu():
-                self._menu_hide_stuff(sub.get_submenu())
-                if not [c for c in sub.get_submenu().get_children() if c.get_property('visible')]:
-                    sub.hide()
+        return "%s (%s)" % statistics.db.get_field(lesson_id, ('title', 'filename'))
     def init_empty_file(self):
         self.m_changed = False
         self.m_savetime = time.time()
@@ -257,15 +193,6 @@ class TrainingSetDialog(gu.EditorDialogBase):
         self.g_ui_manager.get_widget("/ExportToolbar").set_style(gtk.TOOLBAR_BOTH)
     def on_show_help(self, widget):
         self.m_app.handle_href("trainingset-editor.html")
-    def get_trainingset_filenames(self):
-        #FIXME we need to know 100% sure that we will get
-        #unicode strings from this method
-        try:
-            return os.listdir(os.path.join(filesystem.user_data(), "trainingsets"))
-        except OSError, e:
-            if e.errno != 2:
-                print "warning: errno != 2"
-            return []
     def load_file(self, filename):
         p = Dataparser({'yes': True, 'no': False}, {})
         p.parse_file(filename)
@@ -297,11 +224,8 @@ class TrainingSetDialog(gu.EditorDialogBase):
         while iter:
             print >> f, "lesson {"
             lesson_id = self.g_liststore.get_value(iter, self.STORE_LESSON_ID)
-            if self.m_app.lessonfile_manager.get(lesson_id, 'title'):
-                print >> f, "  # %s (%s)" % (
-                    self.m_app.lessonfile_manager.get(lesson_id, 'title'),
-                    self.m_app.lessonfile_manager.get(lesson_id, 'filename')
-                    )
+            print >> f, "  # %s (%s)" % (
+                statistics.db.get_field(lesson_id, ('title', 'filename')))
             print >> f, '  lesson_id = "%s"' % lesson_id
             print >> f, '  count = %i' \
                 % self.g_liststore.get_value(iter, self.STORE_COUNT)
