@@ -19,18 +19,13 @@ import gtk
 import re, math
 import mpd
 import random
-import sys
-import soundcard, cfg
+import cfg
 
 def mangle_email(email):
     email = email.replace("@", " ,, ")
     email = email.replace(".", " ,, ")
     return email
 
-def play_tone(midi_int):
-    soundcard.play_note(cfg.get_int('config/preferred_instrument'),
-                        4, midi_int,
-                        cfg.get_int('config/preferred_instrument_velocity'))
 
 def int_to_intervalname(i, shortname=None, updown=None):
     if shortname:
@@ -260,3 +255,68 @@ def string_get_line_at(s, idx):
             end += 1
     return s[start:end+1].strip("\n")
 
+def new_track():
+    t1 = mpd.Track()
+    t1.set_bpm(cfg.get_int('config/default_bpm'))
+    t1.set_volume(cfg.get_int('config/preferred_instrument_volume'))
+    t1.set_patch(cfg.get_int('config/preferred_instrument'))
+    return t1
+
+def new_percussion_track():
+    t1 = mpd.PercussionTrack()
+    t1.set_bpm(cfg.get_int('config/default_bpm'))
+    t1.set_volume(cfg.get_int('config/preferred_instrument_volume'))
+    return t1
+
+def new_2_tracks():
+    if cfg.get_bool('config/override_default_instrument'):
+        instr_low = cfg.get_int('config/lowest_instrument')
+        instr_low_volume = cfg.get_int('config/lowest_instrument_volume')
+        instr_high = cfg.get_int('config/highest_instrument')
+        instr_high_volume = cfg.get_int('config/highest_instrument_volume')
+    else:
+        instr_low = instr_high = cfg.get_int('config/preferred_instrument')
+        instr_low_volume = instr_high_volume = cfg.get_int('config/preferred_instrument_volume')
+    t1 = mpd.Track()
+    t1.set_bpm(cfg.get_int('config/default_bpm'))
+    t1.set_patch(instr_low)
+    t1.set_volume(instr_low_volume)
+    t2 = mpd.Track()
+    t2.set_patch(instr_high)
+    t2.set_volume(instr_high_volume)
+    return t1, t2
+
+def new_3_tracks():
+    if cfg.get_bool('config/override_default_instrument'):
+        instr_low = cfg.get_int('config/lowest_instrument')
+        instr_low_volume = cfg.get_int('config/lowest_instrument_volume')
+        instr_middle = cfg.get_int('config/middle_instrument')
+        instr_middle_volume = cfg.get_int('config/middle_instrument_volume')
+        instr_high = cfg.get_int('config/highest_instrument')
+        instr_high_volume = cfg.get_int('config/highest_instrument_volume')
+    else:
+        instr_low = instr_middle = instr_high = cfg.get_int('config/preferred_instrument')
+        instr_low_volume = instr_middle_volume = instr_high_volume = cfg.get_int('config/preferred_instrument_volume')
+    t1 = mpd.Track()
+    t1.set_bpm(cfg.get_int('config/default_bpm'))
+    t1.set_patch(instr_low)
+    t1.set_volume(instr_low_volume)
+    t2 = mpd.Track()
+    t2.set_patch(instr_middle)
+    t2.set_volume(instr_middle_volume)
+    t3 = mpd.Track()
+    t3.set_patch(instr_high)
+    t3.set_volume(instr_high_volume)
+    return t1, t2, t3
+
+def play_note(notelen, pitch):
+    mpd.play_note(cfg.get_int('config/preferred_instrument'),
+                  notelen, pitch,
+                  cfg.get_int('config/preferred_instrument_volume'),
+                  velocity=mpd.DEFAULT_VELOCITY)
+
+def play_perc(notelen, pitch):
+    mpd.play_perc(notelen, pitch,
+                  cfg.get_int('config/preferred_instrument_volume'),
+                  velocity=mpd.DEFAULT_VELOCITY)
+            
